
// ------------------------------------------------------------------ Init TOOLBOX and WORKSPACE
var workspace;
function init(toolboxXML, workspaceXML) 
{
    workspace = Blockly.inject('blocklyDiv', {media: '../../media/', toolbox: toolboxXML});
    var wx=Blockly.Xml.textToDom(workspaceXML);
    Blockly.Xml.domToWorkspace(wx, workspace);
    workspace.clearUndo();
}

// ------------------------------------------------------------------ Get WORKSPACE as text
function getWorkspaceAsText()
{
    var dom = Blockly.Xml.workspaceToDom(workspace);
    var txt = Blockly.Xml.domToPrettyText(dom);
    return txt;
}

// ------------------------------------------------------------------ Get TOOLBOX as text
function getToolboxAsText()
{
    var txt = workspace.options.languageTree;
    return txt;
}

// ------------------------------------------------------------------ Get JS-CODE as text
function getBlocklyCode()
{
    var code = Blockly.JavaScript.workspaceToCode(workspace);
    return code;
}

// ------------------------------------------------------------------ LIST VARS
function listVars() 
{
    var s = '';
    var v = workspace.getAllVariables();
    for (var i = 0; i < v.length; i++)
    {
        var name = v[i].name;
        var value = myInterpreter.globalScope.object.properties[name];
        value = value ? value.toString() : '';
        s = s + name + ' = ' + value + '\r\n';
    }
    return s;
}

//function listVars() 
//{
//    var s = '';
//    workspace.getBlocksByType("variables_set", false).forEach(function (item) 
//    {
//        var name = item.getField('VAR').getText();
//        var value = myInterpreter.globalObject.properties[name];
//        value = value ? value.toString() : '';
//        s = s + name + ' = ' + value + '\r\n';
//    });
//    return s;
//}


// ------------------------------------------------------------------ Generate LATEST-CODE
function generateCode()
{
    Blockly.JavaScript.STATEMENT_PREFIX = 'highlightBlock(%1);\n';
    Blockly.JavaScript.addReservedWords('highlightBlock');
    latestCode = Blockly.JavaScript.workspaceToCode(workspace);
}


// ------------------------------------------------------------------ INTERPRETER INIT / RESET
var myInterpreter = null;

function initInterpreter()
{
    if (myInterpreter == null)
    {
        generateCode();
        myInterpreter = new Interpreter(latestCode, initApi);
        addChangeListener();
    }
}

function resetInterpreter()
{
    myInterpreter = null;
}

function addChangeListener()
{
    WorkspaceChanged = false;
    workspace.addChangeListener
    (
        // https://developers.google.com/blockly/guides/configure/web/events
        function (event)
        {
            if (event.isUiEvent) 
            {
                WorkspaceChanged = true;
            }
        }
    );
}


// ------------------------------------------------------------------ RUN SINGLE STEP (Debug = 600 uS/line - Release = 300 uS/line)
var ExecutingWaitBlock = false;
var WorkspaceChanged = false;
var HighlightPause = false;
var RunningManualStep;

function runSingleStep(ManualStep) 
{
    if (myInterpreter)
    {
        RunningManualStep = ManualStep;
        HighlightPause = false;
        do 
        {
            var hasMoreCode = false;
            try 
            {
                hasMoreCode = myInterpreter.step();
            } 
            finally 
            {
                if (WorkspaceChanged) 
                {
                    WorkspaceChanged = false;
                    workspace.highlightBlock(null);
                    return 'PROGRAM_CHANGED';
                }
                 if (!hasMoreCode) 
                {
                    workspace.highlightBlock(null);
                    return 'PROGRAM_END';
                }
            }
        } while (hasMoreCode && !HighlightPause && !ExecutingWaitBlock);
    }
    return "";
}

// ------------------------------------------------------------------ HILIGHT BLOCK
//var HilightedBlock;
function highlightBlock(id)
{
    //HilightedBlock = workspace.getBlockById(id).type;     
    workspace.highlightBlock(id);
    HighlightPause = true;
    ExecutingWaitBlock = false;
}

