﻿
' --------------------------------------------------------------------------------------------
'  REFERENCES
' --------------------------------------------------------------------------------------------
' - Nuget WebView2
' - WebView2 SDK
' - Newtonsoft.Json
' - Update Nuget Package - https://www.nuget.org/packages/System.Threading.Tasks.Extensions
' --------------------------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports System.Threading.Tasks

Public Class Form1

    ' ===================================================================================================
    '  OPEN CLOSE FORM
    ' ===================================================================================================
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ShowWindowTitle()
        InitPanels()
        Load_INI()
        ' ------------------------------------------------- 
        SplitContainer1.Panel1.BorderStyle = BorderStyle.Fixed3D
        SplitContainer1.Panel2.BorderStyle = BorderStyle.Fixed3D
        WV2.Location = New Point(4, 4)
        WV2.Width = SplitContainer1.Panel1.ClientSize.Width - 8
        WV2.Height = SplitContainer1.Panel1.ClientSize.Height - 8
        PrintTextBox.Location = New Point(2, 2)
        PrintTextBox.Width = SplitContainer1.Panel2.ClientSize.Width - 6
        PrintTextBox.Height = SplitContainer1.Panel2.ClientSize.Height - 6
        ' ------------------------------------------------- 
        Load_Program()
        UpdateStartStopButton()
        ShowHideTextAndTools()
        ReadBloclyToolboxFromFile()
        WebBrowser_Start()
        UpdateAllPanels()
        Set_ZOOM()
        ToolTips_Init()
        Timer1.Interval = 20
        Timer1.Start()
        EventsAreEnabled = True
        ' ------------------------------------------------- this makes SendKeys faster
        SchedulerMaxPrecision()
        SetFormOpacity()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Timer1.Stop()
        Save_INI()
        StopRunning()
        BreakRunningLoop = True
        Save_Program(ProgramFile_PathAndName)
        SchedulerDefaultPrecision()
        ExecuteScriptFunction("resetInterpreter()")
        COM_Close()
        If Not Debugger.IsAttached Then
            CloseAllOpenedProcesses()
        End If
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
        ShowHideTextAndTools()
        UpdateAllPanels()
    End Sub

    Friend Sub ShowWindowTitle()
        If ProgramFile_PathAndName <> "" Then
            Text = AppTitleAndVersion() + "  -  Program :  " + IO.Path.GetFileName(ProgramFile_PathAndName)
        Else
            Text = AppTitleAndVersion()
        End If
    End Sub

    Friend Sub ShowProgramTitle()
        ShowHideTextAndTools()
    End Sub

    ' ===================================================================================================
    '  SPLIT CONTAINER - Refresh controls while moving
    ' ===================================================================================================
    Private Sub SplitContainer1_MouseDown(sender As Object, e As MouseEventArgs) Handles SplitContainer1.MouseDown
        SplitContainer1.IsSplitterFixed = True
    End Sub
    Private Sub SplitContainer1_MouseUp(sender As Object, e As MouseEventArgs) Handles SplitContainer1.MouseUp
        SplitContainer1.IsSplitterFixed = False
    End Sub
    Private Sub SplitContainer1_MouseMove(sender As Object, e As MouseEventArgs) Handles SplitContainer1.MouseMove
        If e.Button = MouseButtons.Left Then
            If e.Y > 0 And e.Y < SplitContainer1.Height Then
                SplitContainer1.SplitterDistance = e.Y
                SplitContainer1.Refresh()
            End If
        End If
    End Sub

    ' ===================================================================================================
    '  UNDO / REDO / LOAD / SAVE
    ' ===================================================================================================
    Private Sub Chk_Undo_Click(sender As Object, e As EventArgs) Handles Chk_Undo.Click
        WV2.Select() : WV2.Focus()
        SendKeys.Send("^z")
    End Sub
    Private Sub Chk_Redo_Click(sender As Object, e As EventArgs) Handles Chk_Redo.Click
        WV2.Select() : WV2.Focus()
        SendKeys.Send("^y")
    End Sub
    Private Sub Chk_Load_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Chk_Load.MouseDown
        Chk_Load.Checked = True
        If Running Then StopRunning()
        If e.Button = Windows.Forms.MouseButtons.Left Then
            LoadProgramDialog()
        Else
            Load_Program(ProgramFile_PathAndName)
        End If
        Save_INI()
        Chk_Load.Checked = False
    End Sub
    Private Sub Chk_Save_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Chk_Save.MouseDown
        Chk_Save.Checked = True
        If Running Then StopRunning()
        If e.Button = Windows.Forms.MouseButtons.Left Then
            SaveProgramDialog()
        Else
            Save_Program(ProgramFile_PathAndName)
        End If
        Save_INI()
        Chk_Save.Checked = False
    End Sub

    ' ===================================================================================================
    '  RUN / STEP
    ' ===================================================================================================
    Private Sub Chk_Run_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_Run.CheckedChanged
        If Not EventsAreEnabled Then Return
        Running = Not Running
        If Not Running Then
            AddTextToOutputBox("RUN PAUSED")
            Chk_Continue.BackColor = Color.FromArgb(240, 240, 80)
        End If
        UpdateStartStopButton()
        StartStopRunning()
    End Sub

    Friend Sub UpdateStartStopButton()
        Dim old As Boolean = EventsAreEnabled
        EventsAreEnabled = False
        If Running Then
            Chk_Run.Checked = False
            Chk_Run.Text = "STOP"
        Else
            Chk_Run.Checked = True
            Chk_Run.Text = "RUN"
        End If
        EventsAreEnabled = old
    End Sub

    Friend Sub StartStopRunning(Optional SaveProgram As Boolean = True)
        If Running Then
            ' --------------------------------------------------------
            If SaveProgram Then
                Save_Program(ProgramFile_PathAndName)
            End If
            ' --------------------------------------------------------
            ExecuteScriptFunction("resetInterpreter()")
            ExecuteScriptFunction("initInterpreter()")
            ReceivedText = ""
            AddTextToOutputBox("PROGRAM_START")
            UpdateOutputBox()
            Set_ZOOM()
            ' -------------------------------------------------------- RUN 
            StartRunningTime()
            ExecuteScript()
        Else
            StopRunning()
        End If
    End Sub

    Private Sub Chk_Continue_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_Continue.Click
        Running = True
        Chk_Continue.BackColor = Color.Orange
        UpdateStartStopButton()
        AddTextToOutputBox("PROGRAM_CONTINUED")
        StartRunningTime()
        ExecuteScript()
    End Sub

    Private Sub Chk_Step_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_Step.Click
        If Running Then StopRunning()
        RunSingleStep()
        Chk_Continue.BackColor = Color.FromArgb(240, 240, 80)
    End Sub


    ' ===================================================================================================
    '   HELP / TEST
    ' ===================================================================================================
    Private Sub Btn_Help_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Help.Click
        Process.Start(Application.StartupPath + "\Docs")
    End Sub

    Private Sub Btn_Test_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_Test.Click
        '
        ShowBlocklyCode()
        'ShowBlocklyWorkspace()
    End Sub

    Private Async Sub ShowVariableList()
        If NavigationCompleted Then
            Dim s As String = Await ExecuteScriptFunctionAsync("listVars", New String() {})
            If s <> "" Then txt_Debug.Text = s
            txt_Debug.Refresh()
        End If
    End Sub

    ' ===================================================================================================
    '  WEB BROWSER
    ' ===================================================================================================
    Friend Async Sub WebBrowser_Start()
        Try
            Await WV2.EnsureCoreWebView2Async()
            WV2.CoreWebView2.Settings.IsScriptEnabled = True
            WV2.Source = New Uri(BlocklyFilesPath + "HTML\Blockly.html")
        Catch
        End Try
    End Sub

    Private NavigationCompleted As Boolean = False
    Private Sub WV2_NavigationCompleted(sender As Object,
                                              e As CoreWebView2NavigationCompletedEventArgs) _
                                                                 Handles WV2.NavigationCompleted
        'Await WV2.EnsureCoreWebView2Async()
        SetBlocklyToolboxAndWorkspace()
        Set_ZOOM()
        SetSpeed(tkbar_Speed.Value)
        ' --------------------------------------------------------------------- 
        UpdateStartStopButton()
        ' --------------------------------------------------------------------- RUN AUTOSTART
        If Running Then StartStopRunning(False)
        ' --------------------------------------------------------------------- 
        WV2.CoreWebView2.AddHostObjectToScript("dotnet", New DomManipulationDotnetInterop)
        ' ---------------------------------------------------------------------
        NavigationCompleted = True
    End Sub


    ' ===================================================================================================
    '  TIMER 1 - UPDATE RECEIVED TEXT
    ' ===================================================================================================
    Friend FlagCloseAll As Boolean = False

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        '
        ' --------------------------------------------------------------------- COMMANDS VIA SLOT-TEXT
        ' TestCommandSlot()
        ' --------------------------------------------------------------------- UPDATE txt_Output
        If ReceivedTextChanged Then
            ReceivedTextChanged = False
            UpdateOutputBox()
        End If
        ' --------------------------------------------------------------------- UPDATE Variable List
        If Running AndAlso
            Me.WindowState <> FormWindowState.Minimized AndAlso GroupBox3.Height > 30 Then
            ShowVariableList()
        End If
        ' --------------------------------------------------------------------- CLOSE ALL
        If FlagCloseAll Then
            CloseAllOpenedProcesses()
            StopRunning()
            Close()
        End If
    End Sub

    Private Sub UpdateOutputBox()
        PrintTextBox.Text = ReceivedText
        PrintTextBox.SelectionStart = 9999
        PrintTextBox.ScrollToCaret()
        PrintTextBox.Refresh()
    End Sub


    ' ===================================================================================================
    '  RUNNING START STOP
    ' ===================================================================================================
    Friend Sub StopRunning()
        If Not BreakRunningLoop Then
            BreakRunningLoop = True
        Else
            StopRunning_AndUpdateButton()
        End If
        COM_Close()
        TTS_Stop()
    End Sub

    Private Sub StopRunning_AndUpdateButton()
        Try
            UpdateOutputBox()
            Running = False
            UpdateStartStopButton()
            TestRunningTime()
        Catch
        End Try
    End Sub

    Private ReadOnly SW As New Stopwatch
    Friend Sub StartRunningTime()
        SW.Reset()
        SW.Start()
    End Sub
    Friend Function GetRunningSeconds() As Double
        Return SW.Elapsed.TotalSeconds
    End Function
    Friend Sub TestRunningTime()
        AddTextToOutputBox("Total running time = " + SW.ElapsedMilliseconds.ToString + " mS")
        AddTextToOutputBox("Milliseconds per line = " + (SW.Elapsed.TotalMilliseconds / BlocksCounter).ToString("0.0", GCI))
        UpdateOutputBox()
    End Sub


    ' ===================================================================================================
    '  RUN SINGLE STEP
    ' ===================================================================================================
    Private Async Sub RunSingleStep()
        ExecuteScriptFunction("initInterpreter()")
        StartRunningTime()
        Dim retval As String
        retval = Await ExecuteScriptFunctionAsync("runSingleStep", New String() {"true"})
        ' 
        If retval <> "" Then
            AddTextToOutputBox(retval)
            DecodeCommands_OUTPUT(retval)
        End If
        ' 
        If retval = "PROGRAM_END" Then
            ExecuteScriptFunction("resetInterpreter()")
            ExecuteScriptFunction("initInterpreter()")
            Await ExecuteScriptFunctionAsync("runSingleStep", New String() {"true"})
        End If
        '
        ShowVariableList()
    End Sub

    ' ===================================================================================================
    '  RUNNING LOOP
    ' ===================================================================================================
    Friend BreakRunningLoop As Boolean
    Private BlocksCounter As Int32
    Private Async Sub ExecuteScript()
        BreakRunningLoop = False
        BlocksCounter = 0
        StartRunningTime()
        Do
            Dim retval As String = Await ExecuteScriptFunctionAsync("runSingleStep", New String() {"false"})
            '
            If retval <> "" Then
                AddTextToOutputBox(retval + vbCrLf)
                DecodeCommands_OUTPUT(retval)
            End If
            '
            If LoopMillisec = 0 Then
                If BlocksCounter Mod SleepEveryNloops = 0 Then
                    Threading.Thread.Sleep(1)
                End If
            Else
                If LoopMillisec < 100 Then
                    Threading.Thread.Sleep(LoopMillisec)
                Else
                    Await Task.Delay(LoopMillisec)
                End If
            End If
            '
            BlocksCounter += 1
            '
        Loop Until BreakRunningLoop
        '
        ShowVariableList()
        StopRunning_AndUpdateButton()
    End Sub

    ' ===================================================================================================
    '  JAVASCRIPT CALLBACKS (UNUSED)
    ' ===================================================================================================
    Private Sub WV2_WebMessageReceived(sender As Object,
                                       e As CoreWebView2WebMessageReceivedEventArgs) Handles WV2.WebMessageReceived
        '
        MessageBox.Show("Received unexpected Web-Message", "Theremino Blockly")
    End Sub

    ' ===================================================================================================
    '  HOST OBJECT - TWO WAY FUNCTION CALLS -
    '  Used by the Theremino Blocks: Input / Slot / SlotText
    ' ===================================================================================================
    <ComVisible(True)>
    Public Class DomManipulationDotnetInterop
        ' ---------------------------------------------------------- GET VALUE
        Public Function GetValue(ByVal str As String) As String
            AddTextToOutputBox(str)
            Return DecodeCommands_INPUT(str)
        End Function
        ' ---------------------------------------------------------- SET VALUE
        Public Function SetValue(ByVal str As String) As String
            AddTextToOutputBox(str)
            DecodeCommands_OUTPUT(str)
            Return ""
        End Function
        ' ---------------------------------------------------------- Waiting Synthesizer
        Public Function WaitingSynthesizer() As String
            If Running And Not TTS_SynthesizerReady() Then
                Return "true"
            Else
                Return "false"
            End If
        End Function
    End Class

End Class
