﻿Imports Newtonsoft.Json

Module Module_Blockly

    Friend LoopMillisec As Int32
    Friend SleepEveryNloops As Int32

    ' ===================================================================================================
    '  JAVASCRIPT FUNCTION CALLS
    ' ===================================================================================================
    Friend ReadOnly BlocklyFilesPath As String = Application.StartupPath + "\Blockly\Files\"
    Private ToolboxXML As String
    Friend WorkspaceXML As String

    Friend Async Sub SetBlocklyToolboxAndWorkspace()
        Await ExecuteScriptFunctionAsync("init", New String() {ToolboxXML, WorkspaceXML})
    End Sub

    Friend Sub ReadBloclyToolboxFromFile()
        If IO.File.Exists(BlocklyFilesPath + "XML\blockyToolbox.xml") Then
            ToolboxXML = System.IO.File.ReadAllText(BlocklyFilesPath + "XML\blockyToolbox.xml")
        End If
    End Sub

    Public Sub ExecuteScriptFunction(ByVal FunctionName As String)
        Form1.WV2.ExecuteScriptAsync(FunctionName)
    End Sub

    Public Async Sub ExecuteScriptFunctionAsync(ByVal FunctionName As String)
        Await Form1.WV2.ExecuteScriptAsync(FunctionName)
    End Sub

    Public Async Function ExecuteScriptFunctionAsync(ByVal FunctionName As String,
                                                     ByVal Parameters() As Object) _
                                                     As Threading.Tasks.Task(Of String)
        Dim script As String = FunctionName + "("
        Dim i As Integer = 0
        Do While i < Parameters.Length
            script += JsonConvert.SerializeObject(Parameters(i))
            If i < (Parameters.Length - 1) Then
                script += ", "
            End If
            i += 1
        Loop
        script += ");"

        Dim result As Object = Await Form1.WV2.ExecuteScriptAsync(script)
        result = CType(JsonConvert.DeserializeObject(result.ToString), String)
        Return CStr(result)
    End Function

    Friend Async Sub ShowBlocklyCode()
        Dim result As Object = Await Form1.WV2.ExecuteScriptAsync("getBlocklyCode()")
        result = CType(JsonConvert.DeserializeObject(result.ToString), String)
        MessageBox.Show(result.ToString, "Message from Theremino Blockly")
    End Sub

    Friend Sub ShowBlocklyWorkspace()
        GetBlocklyWorkspace()
        MessageBox.Show(WorkspaceXML, "Message from Theremino Blockly")
    End Sub

    'Friend Async Sub HighlightBlock(id As String)
    '    Dim s(0) As String
    '    s(0) = id
    '    Await ExecuteScriptFunctionAsync("highlightBlock", s)
    'End Sub

    ' ===================================================================================================
    '  GET BLOCKLY WORKSPACE
    ' ===================================================================================================
    '  With a workaround to get the updated workspace (otherwise last editing are loosed) 
    ' ===================================================================================================
    Friend Sub GetBlocklyWorkspace()
        For i As Int32 = 1 To 50 ' <-- timeout 5 seconds
            GetBlocklyWorkspace2()
            Application.DoEvents()
            Threading.Thread.Sleep(100)
            If WorkspaceXML IsNot Nothing AndAlso
                WorkspaceXML.StartsWith("<xml xmlns=") AndAlso
                WorkspaceXML.EndsWith("</xml>") Then
                'Form1.Text = i.ToString
                'Console.Beep(1500, 25)
                Click2()
                Return
            End If
        Next
        WorkspaceXML = ""
        Form1.BreakRunningLoop = True
    End Sub
    Private Async Sub GetBlocklyWorkspace2()
        WorkspaceXML = ""
        Try
            Click1c()
            Dim s As String
            s = Await Form1.WV2.ExecuteScriptAsync("getWorkspaceAsText()")
            s = CType(JsonConvert.DeserializeObject(s), String)
            WorkspaceXML = s
        Catch
        End Try
    End Sub

    ' ===================================================================================================
    '  GET BLOCKLY TOOLBOX
    ' ===================================================================================================
    Friend Async Sub GetBlocklyToolbox()
        Dim s As String
        s = Await Form1.WV2.ExecuteScriptAsync("getToolboxAsText()")
        s = CType(JsonConvert.DeserializeObject(s), String)
        MessageBox.Show(s + vbCrLf + vbCrLf + s.Length.ToString)
    End Sub

    ' ===================================================================================================
    '  ADD TEXT TO OUTPUT BOX
    ' ===================================================================================================
    Friend ReceivedTextChanged As Boolean
    Friend ReceivedText As String
    Friend Sub AddTextToOutputBox(txt As String)
        If txt.ToLower.StartsWith("print ") Then
            If Not Form1.Chk_ShowDebugMessages.Checked Then
                txt = txt.Substring(6)
            End If
        Else
            If Not Form1.Chk_ShowDebugMessages.Checked Then Return
            txt = "DEBUG: " + txt
        End If
        ' ---------------------------------------------------------------------
        ReceivedText = Strings.Right(ReceivedText, 2000) + txt + vbCrLf
        ReceivedTextChanged = True
    End Sub

    ' ====================================================
    '  FAST APPEND To Long STRINGS With String-BUILDER
    ' ====================================================
    'Dim sb As StringBuilder = New StringBuilder
    'sb.Append(StrDup(Indent, " ") + str(n) + vbCrLf)
    'Return sb.ToString

End Module
