﻿Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture


    ' =======================================================================================================
    '   Files
    ' =======================================================================================================
    Private Function FileIsInAppFolders(ByVal filename As String) As Boolean
        If filename.StartsWith(Application.StartupPath, StringComparison.InvariantCultureIgnoreCase) Then
            Return True
        Else
            Return False
        End If
    End Function
    Public Function FolderExists(ByVal FolderName As String) As Boolean
        If FolderName.Length < 2 Then Return False
        FolderName = LCase(FolderName)
        Select Case FolderName
            Case "a:\", "b:\", "c:\", "d:\", "e:\", "f:\", "g:\", "h:\", "i:\", "j:\", "k:\"
                Return True
        End Select
        Return My.Computer.FileSystem.DirectoryExists(FolderName)
    End Function
    Public Function GetPath(ByVal str As String) As String
        Try
            Return IO.Path.GetDirectoryName(str) & "\"
        Catch
            Return str
        End Try
    End Function

    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Friend Sub LimitFormPosition_CompletelyVisible(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
            .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
            .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
            .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
            .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
        End With
    End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 200)         ' not beyond right border - 200 pixels
            .Top = Math.Min(.Top, SB.Bottom - 150)          ' not beyond bottom border - 150 pixels
            .Left = Math.Max(.Left, SB.Left + 200 - .Width) ' at least at left border + 200 pixels
            .Top = Math.Max(.Top, SB.Top)                   ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                               Optional ByVal Value As Double = Double.NaN, _
                               Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt, GCI)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Friend Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Friend Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Friend Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = s.Trim
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim iniFileName As String = Application.StartupPath + "\" + appname + "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            f.WriteLine(TabString("SplitterDistance", Form1.SplitContainer1.SplitterDistance))
            '

            'f.WriteLine(TabString("ProgramFile", ProgramFile_PathAndName))
            f.WriteLine(TabString("ProgramFile", GetPathStartingFromApp(ProgramFile_PathAndName)))
            '
            f.WriteLine(TabString("Running", Running.ToString))
            f.WriteLine(TabString("ShowDebugMessages", Form1.Chk_ShowDebugMessages.Checked))
            '
            f.WriteLine(TabString("GroupBox1_Height", Form1.GroupBox1.Height))
            f.WriteLine(TabString("GroupBox2_Height", Form1.GroupBox2.Height))
            f.WriteLine(TabString("GroupBox3_Height", Form1.GroupBox3.Height))
            '
            f.WriteLine(TabString("Speed", Form1.tkbar_Speed.Value))
            f.WriteLine(TabString("Zoom", Form1.tkbar_Zoom.Value))
            f.WriteLine(TabString("Transparency", Form1.tkbar_Transparency.Value))
            '
            'f.WriteLine(TabString("TimeAsUTC", Form1.chk_TimeAsUTC.Checked))
            'f.WriteLine(TabString("FirstSlot", Form1.txt_FirstSlot.Text))
            'f.WriteLine(TabString("CommandsSlot", Form1.txt_CommandsSlot.Text))
            'f.WriteLine(TabString("AverageSec", Form1.txt_AverageSec.Text))
            '
            'f.WriteLine(TabString("PositionLat", Form1.txt_PositionLat.Text))
            'f.WriteLine(TabString("PositionLon", Form1.txt_PositionLon.Text))
            'f.WriteLine(TabString("PositionAlt", Form1.txt_PositionAlt.Text))
            '
            f.WriteLine(TabString("MapHsize", Form1.txt_MapHsize.Text))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub

    Friend Sub Load_INI()
        ' ------------------------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim iniFileName As String = Application.StartupPath + "\" + appname + "_INI.txt"
        '
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            '
            Dim fs As IO.FileStream
            Dim sr As IO.StreamReader
            fs = New IO.FileStream(iniFileName,
                                   IO.FileMode.Open,
                                   IO.FileAccess.Read,
                                   IO.FileShare.ReadWrite)
            sr = New IO.StreamReader(fs)
            '
            Do While Not sr.EndOfStream
                l = sr.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = Val_Int(l)
                    Case "Form1_Left" : Form1.Left = Val_Int(l)
                    Case "Form1_Width" : Form1.Width = Val_Int(l)
                    Case "Form1_Height" : Form1.Height = Val_Int(l)
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                    Case "SplitterDistance" : Form1.SplitContainer1.SplitterDistance = CInt(Val(l))
                        ' ---------------------------------------------------------------

                    'Case "ProgramFile" : ProgramFile_PathAndName = l
                    Case "ProgramFile" : ProgramFile_PathAndName = SetPathStartingFromApp(l)
                        '
                    Case "Running" : Running = l = "True"
                    Case "ShowDebugMessages" : Form1.Chk_ShowDebugMessages.Checked = l = "True"
                        ' ---------------------------------------------------------------
                    Case "GroupBox1_Height" : Form1.GroupBox1.Height = Val_Int(l)
                    Case "GroupBox2_Height" : Form1.GroupBox2.Height = Val_Int(l)
                    Case "GroupBox3_Height" : Form1.GroupBox3.Height = Val_Int(l)
                        ' ---------------------------------------------------------------
                    Case "Speed" : Form1.tkbar_Speed.Value = Val_Int(l)
                    Case "Zoom" : Form1.tkbar_Zoom.Value = Val_Int(l)
                    Case "Transparency" : Form1.tkbar_Transparency.Value = Val_Int(l)
                        ' ---------------------------------------------------------------
                        'Case "TimeAsUTC" : Form1.chk_TimeAsUTC.Checked = l = "True"
                        'Case "FirstSlot" : Form1.txt_FirstSlot.Text = l
                        'Case "CommandsSlot" : Form1.txt_CommandsSlot.Text = l
                        'Case "AverageSec" : Form1.txt_AverageSec.Text = l
                        '    '
                        'Case "PositionLat" : Form1.txt_PositionLat.Text = l
                        'Case "PositionLon" : Form1.txt_PositionLon.Text = l
                        'Case "PositionAlt" : Form1.txt_PositionAlt.Text = l
                        '
                    Case "MapHsize" : Form1.txt_MapHsize.Text = l
                End Select
            Loop
            If sr IsNot Nothing Then sr.Close()
            If fs IsNot Nothing Then fs.Close()
        End If
        LimitFormPosition(Form1)
    End Sub


    Private Function GetPathStartingFromApp(s As String) As String
        Return ".." + s.Replace(Application.StartupPath, "")
    End Function

    Private Function SetPathStartingFromApp(s As String) As String
        If s.StartsWith("..\") Then
            Return Application.StartupPath + s.TrimStart("."c).TrimStart("."c)
        Else
            Return s
        End If
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- PROGRAMS
    ' ==================================================================================================
    Friend ProgramFile_PathAndName As String = ""

    Friend Sub Save_Program(Optional ByVal fileName As String = "")
        '
        If fileName = "" Then
            fileName = Application.StartupPath + "\Programs\NoName.txt"
        End If
        My.Computer.FileSystem.CreateDirectory(Application.StartupPath + "\Programs")
        '
        Try
            GetBlocklyWorkspace()
            If WorkspaceXML = "" Then
                Exit Sub
            End If
            ' ----------------------------------------------------------- 
            IO.File.WriteAllText(fileName, WorkspaceXML, System.Text.Encoding.Unicode)
            ' ----------------------------------------------------------- 
            ProgramFile_PathAndName = fileName
            Form1.ShowProgramTitle()
            Form1.ShowWindowTitle()
        Catch
        End Try
        ' ----------------------------------------------- remove the enter if used to execute the dialog
        WaitKeyOff(Keys.Enter)
        SendKeys.Flush()
    End Sub

    Friend Sub Load_Program(Optional ByVal fileName As String = "")
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        If fileName = "" Then fileName = ProgramFile_PathAndName
        ' -------------------------------------------------------------------------------
        If My.Computer.FileSystem.FileExists(fileName) Then
            If Not FileIsInAppFolders(fileName) Then
                MsgBox("Warning can not run programs outside the application folders" + vbCrLf + vbCrLf +
                       "The file will be copied in the ""Programs"" folder")
                ' -----------------------------------------------------------------------
                '  test the path and eventually copy it in the same subfolder of "programs"
                ' -----------------------------------------------------------------------
                Dim destname As String
                Dim i As Int32 = InStr(fileName, "Theremino_Automation\Programs\", CompareMethod.Text)
                If i > 0 Then
                    destname = Application.StartupPath + "\Programs\" + Mid(fileName, i + 30)
                Else
                    destname = Application.StartupPath + "\Programs\" + IO.Path.GetFileName(fileName)
                End If
                ' -----------------------------------------------------------------------
                IO.File.Copy(fileName, destname)
                fileName = destname
            End If
        Else
            fileName = FindFileInAppFolders(fileName)
        End If
        ' -------------------------------------------------------------------------------
        If My.Computer.FileSystem.FileExists(fileName) Then
            ' ----------------------------------------------- 
            WorkspaceXML = IO.File.ReadAllText(fileName)
            ' ----------------------------------------------- 
            ProgramFile_PathAndName = fileName
            Form1.ShowProgramTitle()
            Form1.ShowWindowTitle()
            ' ----------------------------------------------- remove the enter key used to execute the dialog
            WaitKeyOff(Keys.Enter)
            SendKeys.Flush()
            ' ----------------------------------------------- clear the old workspace and reload
            If Form1.WV2.CoreWebView2 IsNot Nothing Then
                Form1.WV2.CoreWebView2.Reload()
                SetBlocklyToolboxAndWorkspace()
            End If
        End If
    End Sub

    Private Function FindFileInAppFolders(ByVal fname As String) As String
        If fname = "" Then Return ""
        fname = IO.Path.GetFileName(fname)
        Dim FileNames As Collections.ObjectModel.ReadOnlyCollection(Of String)
        Dim wildcards() As String = {"*.txt"}
        Dim s As String = Application.StartupPath & "\"
        FileNames = My.Computer.FileSystem.GetFiles(s, FileIO.SearchOption.SearchAllSubDirectories, wildcards)
        For Each str As String In FileNames
            If str.EndsWith(fname) Then
                Return str
            End If
        Next
        Return ""
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- PROGRAMS -- DIALOGS
    ' ==================================================================================================
    Friend Sub SaveProgramDialog()
        Dim sfd As New SaveFileDialog()
        sfd.InitialDirectory = PrepareSaveLoadFolder()
        sfd.FileName = IO.Path.GetFileName(ProgramFile_PathAndName)
        'SFD.RestoreDirectory = False
        sfd.AutoUpgradeEnabled = True
        sfd.Title = "Save program file"
        sfd.ValidateNames = False
        sfd.DefaultExt = ".txt"
        sfd.AddExtension = True
        sfd.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*"
        '
        Try ' this TryCatch protects against unbalanced "(" and ")" characters in the file name
            SendKeys.Send("+{TAB}+{TAB}" + sfd.FileName + "{TAB}{TAB}") ' file selected in the list
            'SendKeys.Send("{HOME}{END}^A") ' name visible and selected
            SendKeys.Send("{HOME}{END}")   ' name visible and unselected
        Catch
        End Try
        '
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ' ---------------------------------------- save the old file before to change name and save the new file
            Save_Program(ProgramFile_PathAndName)
            ' ---------------------------------------- save the new file
            Save_Program(sfd.FileName)
        End If
    End Sub

    Friend Sub LoadProgramDialog()
        'If Not Editor_OkToGo("File open") Then Exit Sub
        Dim ofd As New OpenFileDialog()
        ofd.InitialDirectory = PrepareSaveLoadFolder()
        ofd.FileName = IO.Path.GetFileName(ProgramFile_PathAndName)
        'OFD.RestoreDirectory = True
        ofd.AutoUpgradeEnabled = True
        ofd.Title = "Load program file"
        ofd.ValidateNames = False
        ofd.Multiselect = True
        ofd.Filter = "All files (*.*)|*.*"
        ofd.DefaultExt = ".txt"
        '
        Try ' this TryCatch protects against unbalanced "(" and ")" characters in the file name
            SendKeys.Send("+{TAB}+{TAB}" + ofd.FileName + "{TAB}{TAB}") ' file selected in the list
            'SendKeys.Send("{HOME}{END}^A") ' name visible and selected
            SendKeys.Send("{HOME}{END}")   ' name visible and unselected
        Catch
        End Try
        '
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_Program(ProgramFile_PathAndName)
            ProgramFile_PathAndName = ofd.FileName
            Load_Program(ProgramFile_PathAndName)
        End If
    End Sub

    Private Function PrepareSaveLoadFolder() As String
        PrepareSaveLoadFolder = ""
        If My.Computer.FileSystem.FileExists(ProgramFile_PathAndName) Then
            PrepareSaveLoadFolder = GetPath(ProgramFile_PathAndName)
        End If
        If Not FolderExists(PrepareSaveLoadFolder) Or Not FileIsInAppFolders(PrepareSaveLoadFolder) Then
            PrepareSaveLoadFolder = Windows.Forms.Application.StartupPath & "\Programs"
        End If
        PrepareSaveLoadFolder = PlatformAdjustedFileName(PrepareSaveLoadFolder)
    End Function

End Module
